@@ -46,31 +46,47 @@ def write_default_ansible_cfg(work_dir,
     ansible_config_path = os.path.join(work_dir, 'ansible.cfg')
     shutil.copy(base_ansible_cfg, ansible_config_path)
 
-    modules_path = ('/root/.ansible/plugins/modules:'
-                    '/usr/share/ansible/tripleo-plugins/modules:'
-                    '/usr/share/ansible/plugins/modules:'
-                    '%s/library' % constants.DEFAULT_VALIDATIONS_BASEDIR)
+    modules_path = (
+        '/root/.ansible/plugins/modules:'
+        '/usr/share/ansible/tripleo-plugins/modules:'
+        '/usr/share/ansible/plugins/modules:'
+        '{}/library:'
+        '{}/library'.format(constants.DEFAULT_VALIDATIONS_BASEDIR,
+                            constants.DEFAULT_VALIDATIONS_LEGACY_BASEDIR))
     lookups_path = (
         '/root/.ansible/plugins/lookup:'
         '/usr/share/ansible/tripleo-plugins/lookup:'
         '/usr/share/ansible/plugins/lookup:'
-        '%s/lookup_plugins' % constants.DEFAULT_VALIDATIONS_BASEDIR)
+        '{}/lookup_plugins:'
+        '{}/lookup_plugins'.format(
+            constants.DEFAULT_VALIDATIONS_BASEDIR,
+            constants.DEFAULT_VALIDATIONS_LEGACY_BASEDIR))
     callbacks_path = (
         '~/.ansible/plugins/callback:'
         '/usr/share/ansible/tripleo-plugins/callback:'
         '/usr/share/ansible/plugins/callback:'
-        '%s/callback_plugins' % constants.DEFAULT_VALIDATIONS_BASEDIR)
-    callbacks_whitelist = ','.join(['tripleo', 'profile_tasks'])
+        '{}/callback_plugins:'
+        '{}/callback_plugins'.format(
+            constants.DEFAULT_VALIDATIONS_BASEDIR,
+            constants.DEFAULT_VALIDATIONS_LEGACY_BASEDIR))
+    callbacks_whitelist = ','.join(['tripleo_dense', 'tripleo_profile_tasks',
+                                    'tripleo_states'])
     action_plugins_path = (
         '~/.ansible/plugins/action:'
         '/usr/share/ansible/plugins/action:'
         '/usr/share/ansible/tripleo-plugins/action:'
-        '%s/action_plugins' % constants.DEFAULT_VALIDATIONS_BASEDIR)
+        '{}/action_plugins:'
+        '{}/action_plugins'.format(
+            constants.DEFAULT_VALIDATIONS_BASEDIR,
+            constants.DEFAULT_VALIDATIONS_LEGACY_BASEDIR))
     filter_plugins_path = (
         '~/.ansible/plugins/filter:'
         '/usr/share/ansible/plugins/filter:'
         '/usr/share/ansible/tripleo-plugins/filter:'
-        '%s/filter_plugins' % constants.DEFAULT_VALIDATIONS_BASEDIR)
+        '{}/filter_plugins:'
+        '{}/filter_plugins'.format(
+            constants.DEFAULT_VALIDATIONS_BASEDIR,
+            constants.DEFAULT_VALIDATIONS_LEGACY_BASEDIR))
     roles_path = ('%(work_dir)s/roles:'
                   '/root/.ansible/roles:'
                   '/usr/share/ansible/tripleo-roles:'
@@ -79,7 +95,8 @@ def write_default_ansible_cfg(work_dir,
                   '%(ooo_val_path)s/roles:'
                   '%(work_dir)s' % {
                       'work_dir': work_dir,
-                      'ooo_val_path': constants.DEFAULT_VALIDATIONS_BASEDIR
+                      'ooo_val_path':
+                          constants.DEFAULT_VALIDATIONS_LEGACY_BASEDIR
                   })
 
     config = configparser.ConfigParser()
@@ -90,7 +107,7 @@ def write_default_ansible_cfg(work_dir,
     config.set('defaults', 'library', modules_path)
     config.set('defaults', 'callback_plugins', callbacks_path)
     config.set('defaults', 'callback_whitelist', callbacks_whitelist)
-    config.set('defaults', 'stdout_callback', 'tripleo')
+    config.set('defaults', 'stdout_callback', 'tripleo_dense')
     config.set('defaults', 'action_plugins', action_plugins_path)
     config.set('defaults', 'lookup_plugins', lookups_path)
     config.set('defaults', 'filter_plugins', filter_plugins_path)
@@ -102,7 +119,8 @@ def write_default_ansible_cfg(work_dir,
                     datetime.now().strftime("%Y-%m-%dT%H:%M:%S"))
         os.rename(log_path, new_path)
 
-    config.set('defaults', 'forks', str(multiprocessing.cpu_count() * 10))
+    config.set('defaults', 'forks', str(min(
+        multiprocessing.cpu_count() * 4, 100)))
     config.set('defaults', 'timeout', '30')
     config.set('defaults', 'gather_timeout', '30')
 
@@ -148,7 +166,10 @@ def write_default_ansible_cfg(work_dir,
         sio_cfg = StringIO()
         sio_cfg.write(override_ansible_cfg)
         sio_cfg.seek(0)
-        config.readfp(sio_cfg)
+        if sys.version_info[0] < 3:
+            config.readfp(sio_cfg)  # pylint: disable=deprecated-method
+        else:
+            config.read_file(sio_cfg)
         sio_cfg.close()
 
     with open(ansible_config_path, 'w') as configfile:
@@ -392,7 +413,7 @@ class AnsiblePlaybookAction(base.TripleOAction):
         self.trash_output = self._kwargs_for_run.pop('trash_output', False)
         self.profile_tasks = self._kwargs_for_run.pop('profile_tasks', True)
         self.profile_tasks_limit = self._kwargs_for_run.pop(
-            'profile_tasks_limit', 0)
+            'profile_tasks_limit', 20)
         self.blacklisted_hostnames = self._kwargs_for_run.pop(
             'blacklisted_hostnames', [])
         self.override_ansible_cfg = self._kwargs_for_run.pop(
@@ -520,15 +541,21 @@ class AnsiblePlaybookAction(base.TripleOAction):
         python_version = sys.version_info.major
         ansible_playbook_cmd = "ansible-playbook-{}".format(python_version)
 
-        if 0 < self.verbosity < 6:
-            verbosity_option = '-' + ('v' * self.verbosity)
+        if 1 < self.verbosity < 6:
+            verbosity_option = '-' + ('v' * (self.verbosity - 1))
             command = [ansible_playbook_cmd, verbosity_option,
                        self.playbook]
         else:
             command = [ansible_playbook_cmd, self.playbook]
 
+        # --limit should always take precedence over blacklisted hosts.
+        # https://bugzilla.redhat.com/show_bug.cgi?id=1857298
         if self.limit_hosts:
             command.extend(['--limit', self.limit_hosts])
+        elif self.blacklisted_hostnames:
+            host_pattern = ':'.join(
+                ['!%s' % h for h in self.blacklisted_hostnames if h])
+            command.extend(['--limit', host_pattern])
 
         if self.module_path:
             command.extend(['--module-path', self.module_path])
@@ -560,11 +587,6 @@ class AnsiblePlaybookAction(base.TripleOAction):
         if self.inventory:
             command.extend(['--inventory-file', self.inventory])
 
-        if self.blacklisted_hostnames:
-            host_pattern = ':'.join(
-                ['!%s' % h for h in self.blacklisted_hostnames if h])
-            command.extend(['--limit', host_pattern])
-
         if self.tags:
             command.extend(['--tags', self.tags])
 
@@ -598,8 +620,9 @@ class AnsiblePlaybookAction(base.TripleOAction):
                 env_variables.update({
                     # the whitelist could be collected from multiple
                     # arguments if we find a use case for it
-                    'ANSIBLE_CALLBACK_WHITELIST': 'tripleo,profile_tasks',
-                    'ANSIBLE_STDOUT_CALLBACK': 'tripleo',
+                    'ANSIBLE_CALLBACK_WHITELIST':
+                        'tripleo_dense,tripleo_profile_tasks,tripleo_states',
+                    'ANSIBLE_STDOUT_CALLBACK': 'tripleo_dense',
                     'PROFILE_TASKS_TASK_OUTPUT_LIMIT':
                         six.text_type(self.profile_tasks_limit),
                 })
@@ -654,7 +677,7 @@ class AnsiblePlaybookAction(base.TripleOAction):
                     lines.append(line)
                     if not self.trash_output:
                         stdout.append(line)
-                    if time.time() - start > 30:
+                    if time.time() - start > 5:
                         self.post_message(queue, ''.join(lines))
                         lines = []
                         start = time.time()
