@@ -39,16 +39,10 @@ class J2SwiftLoader(jinja2.BaseLoader):
     only the absolute path relative to the container root is searched.
     """
 
-    def __init__(self, swift, container, searchpath=None):
+    def __init__(self, swift, container, searchpath):
         self.swift = swift
         self.container = container
-        if searchpath is not None:
-            if isinstance(searchpath, six.string_types):
-                self.searchpath = [searchpath]
-            else:
-                self.searchpath = list(searchpath)
-        else:
-            self.searchpath = []
+        self.searchpath = [searchpath]
         # Always search the absolute path from the root of the swift container
         if '' not in self.searchpath:
             self.searchpath.append('')
@@ -99,15 +93,9 @@ class ProcessTemplatesAction(base.TripleOAction):
     def __init__(self, container=constants.DEFAULT_CONTAINER_NAME):
         super(ProcessTemplatesAction, self).__init__()
         self.container = container
+        self.role_data = None
 
-    def _j2_render_and_put(self,
-                           j2_template,
-                           j2_data,
-                           outfile_name=None,
-                           context=None):
-        swift = self.get_object_client(context)
-        yaml_f = outfile_name or j2_template.replace('.j2.yaml', '.yaml')
-
+    def _j2_render_and_put(self, j2_template, j2_data, yaml_f, swift):
         # Search for templates relative to the current template path first
         template_base = os.path.dirname(yaml_f)
         j2_loader = J2SwiftLoader(swift, self.container, template_base)
@@ -287,7 +275,7 @@ class ProcessTemplatesAction(base.TripleOAction):
                             self._j2_render_and_put(j2_template,
                                                     j2_data,
                                                     out_f_path,
-                                                    context=context)
+                                                    swift)
                         else:
                             # Backwards compatibility with templates
                             # that specify {{role}} vs {{role.name}}
@@ -296,12 +284,12 @@ class ProcessTemplatesAction(base.TripleOAction):
                             self._j2_render_and_put(j2_template,
                                                     j2_data,
                                                     out_f_path,
-                                                    context=context)
+                                                    swift)
                     else:
                         LOG.info("Skipping rendering of %s, defined in %s" %
                                  (out_f_path, j2_excl_data))
 
-            elif (f.endswith('.network.j2.yaml')):
+            elif f.endswith('.network.j2.yaml'):
                 LOG.info("jinja2 rendering network template %s" % f)
                 j2_template = swiftutils.get_object_string(swift,
                                                            self.container,
@@ -323,7 +311,7 @@ class ProcessTemplatesAction(base.TripleOAction):
                         self._j2_render_and_put(j2_template,
                                                 j2_data,
                                                 out_f_path,
-                                                context=context)
+                                                swift)
                     else:
                         LOG.info("Skipping rendering of %s, defined in %s" %
                                  (out_f_path, j2_excl_data))
@@ -338,7 +326,8 @@ class ProcessTemplatesAction(base.TripleOAction):
                 self._j2_render_and_put(j2_template,
                                         j2_data,
                                         out_f,
-                                        context=context)
+                                        swift)
+        return role_data
 
     def run(self, context):
         error_text = None
@@ -360,7 +349,7 @@ class ProcessTemplatesAction(base.TripleOAction):
             # not found in swift, but if they are found and an exception
             # occurs during processing, that exception will cause the
             # ProcessTemplatesAction to return an error result.
-            self._process_custom_roles(context)
+            self.role_data = self._process_custom_roles(context)
         except Exception as err:
             LOG.exception("Error occurred while processing custom roles.")
             return actions.Result(error=six.text_type(err))
