@@ -16,7 +16,8 @@
 # under the License.
 
 from collections import OrderedDict
-import os.path
+import os
+import tempfile
 import yaml
 
 
@@ -86,7 +87,7 @@ class TripleoInventories(object):
                             inventory[new_key] = {
                                 'children': {x: {} for x in roles}
                             }
-                        if vars in inv[key]:
+                        if 'vars' in inv[key]:
                             inventory[new_key]['vars'] = inv[key]['vars']
                         if key == 'allovercloud':
                             # useful to have just stack name refer to children
@@ -137,8 +138,14 @@ class TripleoInventories(object):
                 if var in inventory:
                     inventory[var]['vars'].update(value)
 
-        with open(inventory_file_path, 'w') as inventory_file:
+        # Atomic update as concurrent tripleoclient commands can call this
+        inventory_file_dir = os.path.dirname(inventory_file_path)
+        with tempfile.NamedTemporaryFile(
+                'w',
+                dir=inventory_file_dir,
+                delete=False) as inventory_file:
             yaml.dump(inventory, inventory_file, TemplateDumper)
+        os.rename(inventory_file.name, inventory_file_path)
 
     def host(self):
         # Dynamic inventory scripts must return empty json if they don't
