@@ -14,11 +14,13 @@
 #
 
 
+import copy
 import mock
 import os
 import six
 import sys
 import tempfile
+import unittest
 import yaml
 
 from tripleo_common import constants
@@ -29,7 +31,7 @@ from tripleo_common.tests import base
 
 TEMPLATE_PATH = os.path.join(os.path.dirname(__file__),
                              '..', '..', '..', 'container-images',
-                             'overcloud_containers.yaml.j2')
+                             'tripleo_containers.yaml.j2')
 
 
 DEFAULTS_PATH = os.path.join(os.path.dirname(__file__),
@@ -213,40 +215,55 @@ class TestKollaImageBuilderTemplate(base.TestCase):
             with open(imagefile.name, 'w') as f:
                 f.write(template_filedata)
 
-    def test_container_images_from_template(self):
+    @mock.patch.object(sys, 'version_info')
+    def test_container_images_from_template(self, mock_ver):
+        mock_ver.major = 2
+        original = kb.CONTAINER_IMAGES_DEFAULTS
+        defs = copy.deepcopy(original)
+        defs['namespace'] = 'docker.io/tripleotrain'
+        kb.CONTAINER_IMAGES_DEFAULTS = defs
         builder = kb.KollaImageBuilder(self.filelist)
         result = builder.container_images_from_template(
             push_destination='localhost:8787',
             tag='liberty'
         )
+        kb.CONTAINER_IMAGES_DEFAULTS = original
         # template substitution on the container_images_template section should
         # be identical to the container_images section
         container_images = yaml.safe_load(filedata)['container_images']
         self.assertEqual(container_images, result)
 
-    def test_container_images_template_inputs(self):
+    @mock.patch.object(sys, 'version_info')
+    def test_container_images_template_inputs(self, mock_ver):
+        mock_ver.major = 2
         builder = kb.KollaImageBuilder(self.filelist)
         self.assertEqual(
             kb.CONTAINER_IMAGES_DEFAULTS,
             builder.container_images_template_inputs()
         )
 
+        original = kb.CONTAINER_IMAGES_DEFAULTS
+        defs = copy.deepcopy(original)
+        defs['namespace'] = 'docker.io/tripleotrain'
+        kb.CONTAINER_IMAGES_DEFAULTS = defs
+        result = builder.container_images_template_inputs()
+        kb.CONTAINER_IMAGES_DEFAULTS = original
         self.assertEqual(
             {
                 'namespace': 'docker.io/tripleotrain',
-                'ceph_namespace': 'docker.io/ceph',
+                'ceph_namespace': 'quay.ceph.io/ceph-ci',
                 'ceph_image': 'daemon',
-                'ceph_tag': 'v4.0.12-stable-4.0-nautilus-centos-7-x86_64',
-                'ceph_grafana_namespace': 'docker.io/grafana',
+                'ceph_tag': 'v4.0.13-stable-4.0-nautilus-centos-7-x86_64',
+                'ceph_grafana_namespace': 'quay.io/app-sre',
                 'ceph_grafana_image': 'grafana',
                 'ceph_grafana_tag': '5.2.4',
-                'ceph_prometheus_namespace': 'docker.io/prom',
+                'ceph_prometheus_namespace': 'quay.io/prometheus',
                 'ceph_prometheus_image': 'prometheus',
                 'ceph_prometheus_tag': 'v2.7.2',
-                'ceph_alertmanager_namespace': 'docker.io/prom',
+                'ceph_alertmanager_namespace': 'quay.io/prometheus',
                 'ceph_alertmanager_image': 'alertmanager',
                 'ceph_alertmanager_tag': 'v0.16.2',
-                'ceph_node_exporter_namespace': 'docker.io/prom',
+                'ceph_node_exporter_namespace': 'quay.io/prometheus',
                 'ceph_node_exporter_image': 'node-exporter',
                 'ceph_node_exporter_tag': 'v0.17.0',
                 'name_prefix': 'centos-binary-',
@@ -255,25 +272,25 @@ class TestKollaImageBuilderTemplate(base.TestCase):
                 'rhel_containers': False,
                 'neutron_driver': 'ovn',
             },
-            builder.container_images_template_inputs()
+            result
         )
 
         self.assertEqual(
             {
                 'namespace': '192.0.2.0:5000/tripleotrain',
-                'ceph_namespace': 'docker.io/cephh',
+                'ceph_namespace': 'quay.ceph.io/ceph-ci',
                 'ceph_image': 'ceph-daemon',
                 'ceph_tag': 'latest',
-                'ceph_grafana_namespace': 'docker.io/grafana',
+                'ceph_grafana_namespace': 'quay.io/app-sre',
                 'ceph_grafana_image': 'grafana',
                 'ceph_grafana_tag': '5.2.4',
-                'ceph_prometheus_namespace': 'docker.io/prom',
+                'ceph_prometheus_namespace': 'quay.io/prometheus',
                 'ceph_prometheus_image': 'prometheus',
                 'ceph_prometheus_tag': 'v2.7.2',
-                'ceph_alertmanager_namespace': 'docker.io/prom',
+                'ceph_alertmanager_namespace': 'quay.io/prometheus',
                 'ceph_alertmanager_image': 'alertmanager',
                 'ceph_alertmanager_tag': 'v0.16.2',
-                'ceph_node_exporter_namespace': 'docker.io/prom',
+                'ceph_node_exporter_namespace': 'quay.io/prometheus',
                 'ceph_node_exporter_image': 'node-exporter',
                 'ceph_node_exporter_tag': 'v0.17.0',
                 'name_prefix': 'prefix-',
@@ -284,7 +301,7 @@ class TestKollaImageBuilderTemplate(base.TestCase):
             },
             builder.container_images_template_inputs(
                 namespace='192.0.2.0:5000/tripleotrain',
-                ceph_namespace='docker.io/cephh',
+                ceph_namespace='quay.ceph.io/ceph-ci',
                 ceph_image='ceph-daemon',
                 ceph_tag='latest',
                 name_prefix='prefix',
@@ -295,7 +312,59 @@ class TestKollaImageBuilderTemplate(base.TestCase):
             )
         )
 
-    def test_container_images_from_template_filter(self):
+    @mock.patch.object(sys, 'version_info')
+    def test_container_images_template_inputs_centos8(self, mock_ver):
+        mock_ver.major = 3
+        builder = kb.KollaImageBuilder(self.filelist)
+
+        original = kb.CONTAINER_IMAGES_DEFAULTS
+        defs = copy.deepcopy(original)
+        defs['namespace'] = 'docker.io/tripleotrain'
+        kb.CONTAINER_IMAGES_DEFAULTS = defs
+        result = builder.container_images_template_inputs(
+            ceph_namespace='quay.ceph.io/ceph-ci',
+            ceph_image='ceph-daemon',
+            ceph_tag='latest',
+            name_prefix='prefix',
+            name_suffix='suffix',
+            tag='master',
+            rhel_containers=False,
+            neutron_driver='ovn')
+        kb.CONTAINER_IMAGES_DEFAULTS = original
+
+        self.assertEqual(
+            {
+                'namespace': 'docker.io/tripleotraincentos8',
+                'ceph_namespace': 'quay.ceph.io/ceph-ci',
+                'ceph_image': 'ceph-daemon',
+                'ceph_tag': 'latest',
+                'ceph_grafana_namespace': 'quay.io/app-sre',
+                'ceph_grafana_image': 'grafana',
+                'ceph_grafana_tag': '5.2.4',
+                'ceph_prometheus_namespace': 'quay.io/prometheus',
+                'ceph_prometheus_image': 'prometheus',
+                'ceph_prometheus_tag': 'v2.7.2',
+                'ceph_alertmanager_namespace': 'quay.io/prometheus',
+                'ceph_alertmanager_image': 'alertmanager',
+                'ceph_alertmanager_tag': 'v0.16.2',
+                'ceph_node_exporter_namespace': 'quay.io/prometheus',
+                'ceph_node_exporter_image': 'node-exporter',
+                'ceph_node_exporter_tag': 'v0.17.0',
+                'name_prefix': 'prefix-',
+                'name_suffix': '-suffix',
+                'tag': 'master',
+                'rhel_containers': False,
+                'neutron_driver': 'ovn',
+            }, result
+        )
+
+    @mock.patch.object(sys, 'version_info')
+    def test_container_images_from_template_filter(self, mock_ver):
+        mock_ver.major = 2
+        original = kb.CONTAINER_IMAGES_DEFAULTS
+        defs = copy.deepcopy(original)
+        defs['namespace'] = 'docker.io/tripleotrain'
+        kb.CONTAINER_IMAGES_DEFAULTS = defs
         builder = kb.KollaImageBuilder(self.filelist)
 
         def filter(entry):
@@ -312,6 +381,7 @@ class TestKollaImageBuilderTemplate(base.TestCase):
             filter=filter,
             tag='liberty'
         )
+        kb.CONTAINER_IMAGES_DEFAULTS = original
         container_images = [{
             'image_source': 'kolla',
             'imagename': 'docker.io/tripleotrain/'
@@ -338,15 +408,15 @@ class TestKollaImageBuilderTemplate(base.TestCase):
     def _test_container_images_yaml_in_sync_helper(self, neutron_driver=None,
                                                    rhel_containers=False,
                                                    remove_images=[]):
-        '''Confirm overcloud_containers.tpl.yaml equals overcloud_containers.yaml
+        '''Confirm tripleo_containers.tpl.yaml equals tripleo_containers.yaml
 
-        TODO(sbaker) remove when overcloud_containers.yaml is deleted
+        TODO(sbaker) remove when tripleo_containers.yaml is deleted
         '''
         mod_dir = os.path.dirname(sys.modules[__name__].__file__)
         project_dir = os.path.abspath(os.path.join(mod_dir, '../../../'))
         files_dir = os.path.join(project_dir, 'container-images')
 
-        oc_tmpl_file = os.path.join(files_dir, 'overcloud_containers.yaml.j2')
+        oc_tmpl_file = os.path.join(files_dir, 'tripleo_containers.yaml.j2')
         tmpl_builder = kb.KollaImageBuilder([oc_tmpl_file])
 
         def ffunc(entry):
@@ -356,82 +426,90 @@ class TestKollaImageBuilderTemplate(base.TestCase):
                 del(entry['services'])
             return entry
 
-        result = tmpl_builder.container_images_from_template(
-            filter=ffunc, neutron_driver=neutron_driver,
-            rhel_containers=rhel_containers)
+        with mock.patch.object(sys, 'version_info') as mock_ver:
+            mock_ver.major = 2
+            result = tmpl_builder.container_images_from_template(
+                filter=ffunc, neutron_driver=neutron_driver,
+                rhel_containers=rhel_containers)
 
-        oc_yaml_file = os.path.join(files_dir, 'overcloud_containers.yaml')
+        oc_yaml_file = os.path.join(files_dir, 'tripleo_containers.yaml')
         yaml_builder = kb.KollaImageBuilder([oc_yaml_file])
         container_images = yaml_builder.load_config_files(
             yaml_builder.CONTAINER_IMAGES)
 
-        # remove image references from overcloud_containers.yaml specified
+        # remove image references from tripleo_containers.yaml specified
         # in remove_images param.
         for image in remove_images:
-            container_images.remove(image)
-
+            try:
+                print("Removing %s" % image)
+                container_images.remove(image)
+            except ValueError:
+                raise ValueError(container_images, image)
         self.assertSequenceEqual(container_images, result)
 
+    @unittest.skipIf(sys.version_info.major > 2, "train defaults are py2")
     def test_container_images_yaml_in_sync(self):
         remove_images = [
             {'image_source': 'kolla',
              'imagename': 'docker.io/tripleotrain/centos-binary'
                           '-neutron-server-opendaylight:current-tripleo'},
-            {'image_source': 'kolla',
+            {'image_source': 'tripleo',
              'imagename': 'docker.io/tripleotrain/centos-binary'
                           '-neutron-server-ovn:current-tripleo'},
             {'image_source': 'kolla',
              'imagename': 'docker.io/tripleotrain/centos-binary'
                           '-opendaylight:current-tripleo'},
-            {'image_source': 'kolla',
+            {'image_source': 'tripleo',
              'imagename': 'docker.io/tripleotrain/centos-binary'
                           '-ovn-northd:current-tripleo'},
-            {'image_source': 'kolla',
+            {'image_source': 'tripleo',
              'imagename': 'docker.io/tripleotrain/centos-binary-ovn-'
                           'controller:current-tripleo'},
-            {'image_source': 'kolla',
+            {'image_source': 'tripleo',
              'imagename': 'docker.io/tripleotrain/centos-binary-ovn-'
                           'nb-db-server:current-tripleo'},
-            {'image_source': 'kolla',
+            {'image_source': 'tripleo',
              'imagename': 'docker.io/tripleotrain/centos-binary-ovn-'
                           'sb-db-server:current-tripleo'},
-            {'image_source': 'kolla',
+            {'image_source': 'tripleo',
              'imagename': 'docker.io/tripleotrain/centos-binary'
                           '-neutron-metadata-agent-ovn:current-tripleo'}]
         self._test_container_images_yaml_in_sync_helper(
             remove_images=remove_images)
 
+    @unittest.skipIf(sys.version_info.major > 2, "train defaults are py2")
     def test_container_images_yaml_in_sync_for_odl(self):
-        # remove neutron-server image reference from overcloud_containers.yaml
+        # remove neutron-server image reference from tripleo_containers.yaml
         remove_images = [
-            {'image_source': 'kolla',
+            {'image_source': 'tripleo',
              'imagename': 'docker.io/tripleotrain/centos-binary'
                           '-neutron-server:current-tripleo'},
-            {'image_source': 'kolla',
+            {'image_source': 'tripleo',
              'imagename': 'docker.io/tripleotrain/centos-binary'
                           '-neutron-server-ovn:current-tripleo'},
-            {'image_source': 'kolla',
+            {'image_source': 'tripleo',
              'imagename': 'docker.io/tripleotrain/centos-binary'
                           '-ovn-northd:current-tripleo'},
-            {'image_source': 'kolla',
+            {'image_source': 'tripleo',
              'imagename': 'docker.io/tripleotrain/centos-binary-ovn-'
                           'controller:current-tripleo'},
-            {'image_source': 'kolla',
+            {'image_source': 'tripleo',
              'imagename': 'docker.io/tripleotrain/centos-binary-ovn-'
                           'nb-db-server:current-tripleo'},
-            {'image_source': 'kolla',
+            {'image_source': 'tripleo',
              'imagename': 'docker.io/tripleotrain/centos-binary-ovn-'
                           'sb-db-server:current-tripleo'},
-            {'image_source': 'kolla',
+            {'image_source': 'tripleo',
              'imagename': 'docker.io/tripleotrain/centos-binary'
                           '-neutron-metadata-agent-ovn:current-tripleo'}]
         self._test_container_images_yaml_in_sync_helper(
             neutron_driver='odl', remove_images=remove_images)
 
+    @unittest.skipIf(sys.version_info.major > 2, "train defaults are py2")
     def test_container_images_yaml_in_sync_for_ovn(self):
-        # remove neutron-server image reference from overcloud_containers.yaml
+        # remove neutron-server image reference from tripleo_containers.yaml
         remove_images = [
-            {'image_source': 'kolla',
+            {'image_source': 'tripleo',
              'imagename': 'docker.io/tripleotrain/centos-binary'
                           '-neutron-server:current-tripleo'},
             {'image_source': 'kolla',
@@ -443,9 +521,10 @@ class TestKollaImageBuilderTemplate(base.TestCase):
         self._test_container_images_yaml_in_sync_helper(
             neutron_driver='ovn', remove_images=remove_images)
 
+    @unittest.skipIf(sys.version_info.major > 2, "train defaults are py2")
     def test_container_images_yaml_in_sync_for_rhel(self):
         remove_images = [
-            {'image_source': 'kolla',
+            {'image_source': 'tripleo',
                 'imagename': 'docker.io/tripleotrain/centos-binary'
                              '-neutron-server:current-tripleo'},
             {'image_source': 'kolla',
@@ -459,13 +538,7 @@ class TestKollaImageBuilderTemplate(base.TestCase):
                              '-fluentd:current-tripleo'},
             {'image_source': 'kolla',
                 'imagename': 'docker.io/tripleotrain/centos-binary'
-                             '-sensu-client:current-tripleo'},
-            {'image_source': 'kolla',
-                'imagename': 'docker.io/tripleotrain/centos-binary'
-                             '-skydive-agent:current-tripleo'},
-            {'image_source': 'kolla',
-                'imagename': 'docker.io/tripleotrain/centos-binary'
-                             '-skydive-analyzer:current-tripleo'}]
+                             '-sensu-client:current-tripleo'}]
         self._test_container_images_yaml_in_sync_helper(
             rhel_containers=True, neutron_driver='ovn',
             remove_images=remove_images)
@@ -524,7 +597,7 @@ class TestPrepare(base.TestCase):
     def test_prepare_simple(self, mock_insecure):
         self.assertEqual({
             'container_images.yaml': [
-                {'image_source': 'kolla',
+                {'image_source': 'tripleo',
                  'imagename': '192.0.2.0:8787/t/p-nova-compute:l'}
             ],
             'environments/containers-default-parameters.yaml': {
@@ -553,7 +626,7 @@ class TestPrepare(base.TestCase):
     def test_prepare_includes(self, mock_insecure):
         self.assertEqual({
             'container_images.yaml': [
-                {'image_source': 'kolla',
+                {'image_source': 'tripleo',
                  'imagename': '192.0.2.0:8787/t/p-nova-libvirt:l'}
             ],
             'environments/containers-default-parameters.yaml': {
@@ -580,7 +653,7 @@ class TestPrepare(base.TestCase):
         # assert same result as includes only. includes trumps excludes
         self.assertEqual({
             'container_images.yaml': [
-                {'image_source': 'kolla',
+                {'image_source': 'tripleo',
                  'imagename': '192.0.2.0:8787/t/p-nova-libvirt:l'}
             ],
             'environments/containers-default-parameters.yaml': {
@@ -607,7 +680,7 @@ class TestPrepare(base.TestCase):
     def test_prepare_push_dest(self, mock_insecure):
         self.assertEqual({
             'container_images.yaml': [{
-                'image_source': 'kolla',
+                'image_source': 'tripleo',
                 'imagename': 'docker.io/t/p-nova-api:l',
                 'push_destination': '192.0.2.0:8787',
             }],
@@ -643,7 +716,7 @@ class TestPrepare(base.TestCase):
         mock_gur.return_value = '192.0.2.0:8787'
         self.assertEqual({
             'container_images.yaml': [{
-                'image_source': 'kolla',
+                'image_source': 'tripleo',
                 'imagename': 'docker.io/t/p-nova-api:l',
                 'push_destination': '192.0.2.0:8787',
             }],
@@ -701,7 +774,7 @@ class TestPrepare(base.TestCase):
     def test_prepare_neutron_driver_default(self, mock_insecure):
         self.assertEqual({
             'container_images.yaml': [
-                {'image_source': 'kolla',
+                {'image_source': 'tripleo',
                  'imagename': 't/p-neutron-server:l'}
             ],
             'environments/containers-default-parameters.yaml': {
@@ -730,9 +803,9 @@ class TestPrepare(base.TestCase):
     def test_prepare_neutron_driver_ovn(self, mock_insecure):
         self.assertEqual({
             'container_images.yaml': [
-                {'image_source': 'kolla',
+                {'image_source': 'tripleo',
                  'imagename': 't/p-neutron-server-ovn:l'},
-                {'image_source': 'kolla',
+                {'image_source': 'tripleo',
                  'imagename': 't/p-ovn-controller:l'}
             ],
             'environments/containers-default-parameters.yaml': {
@@ -794,6 +867,36 @@ class TestPrepare(base.TestCase):
             )
         )
 
+    @mock.patch.object(image_uploader, 'ImageUploadManager')
+    @mock.patch('tripleo_common.image.kolla_builder.'
+                'detect_insecure_registries', return_value={})
+    def test_prepare_default_tag(self, mock_insecure, mock_manager):
+        mock_manager_instance = mock.Mock()
+        mock_manager.return_value = mock_manager_instance
+        mock_uploader = mock.Mock()
+        mock_uploader.discover_image_tags.return_value = []
+        mock_manager_instance.uploader.return_value = mock_uploader
+
+        kb.container_images_prepare(
+            template_file=TEMPLATE_PATH,
+            output_env_file=constants.CONTAINER_DEFAULTS_ENVIRONMENT,
+            output_images_file='container_images.yaml',
+            mapping_args={},
+            tag_from_label="n-v",
+        )
+        self.assertTrue(
+            mock_uploader.discover_image_tags.call_args_list[0][0][2])
+
+        kb.container_images_prepare(
+            template_file=TEMPLATE_PATH,
+            output_env_file=constants.CONTAINER_DEFAULTS_ENVIRONMENT,
+            output_images_file='container_images.yaml',
+            mapping_args={"tag": "master"},
+            tag_from_label="n-v",
+        )
+        self.assertFalse(
+            mock_uploader.discover_image_tags.call_args_list[1][0][2])
+
     def test_get_enabled_services_empty(self):
         self.assertEqual(
             set([]),
@@ -878,13 +981,11 @@ class TestPrepare(base.TestCase):
                 'OS::TripleO::Services::HeatApi',
                 'OS::TripleO::Services::NovaApi',
                 'OS::TripleO::Services::NovaCompute',
+                'OS::TripleO::Services::NeutronApi',
                 'OS::TripleO::Services::Kubernetes::Worker',
-                'OS::TripleO::Services::SkydiveAgent',
             ]),
             kb.build_service_filter({
                 'resource_registry': {
-                    'OS::TripleO::Services::NeutronApi':
-                    '/tht/puppet/services/foo.yaml',
                     'OS::TripleO::Services::NovaApi':
                     '/tht/docker/services/foo.yaml',
                     'OS::TripleO::Services::NovaCompute':
@@ -892,8 +993,6 @@ class TestPrepare(base.TestCase):
                     'OS::TripleO::Services::Kubernetes::Worker':
                     'deployment' +
                     'kubernetes/kubernetes-worker-baremetal-ansible.yaml',
-                    'OS::TripleO::Services::SkydiveAgent':
-                    'deployment/skydive/skydive-agent-baremetal-ansible.yaml',
                     'OS::TripleO::Services::Noop':
                     'OS::Heat::None'
                 }
@@ -905,7 +1004,6 @@ class TestPrepare(base.TestCase):
                         'OS::TripleO::Services::HeatApi',
                         'OS::TripleO::Services::NeutronApi',
                         'OS::TripleO::Services::NovaApi',
-                        'OS::TripleO::Services::SkydiveAgent',
                         'OS::TripleO::Services::Noop'
                     ]
                 }, {
@@ -933,7 +1031,6 @@ class TestPrepare(base.TestCase):
             'namespace': 't',
             'name_prefix': '',
             'name_suffix': '',
-            'tag': 'l',
         }
         env = {
             'parameter_defaults': {
@@ -954,6 +1051,15 @@ class TestPrepare(base.TestCase):
                     'modify_role': 'add-foo-plugin',
                     'modify_only_with_labels': ['kolla_version'],
                     'modify_vars': {'foo_version': '1.0.1'}
+                }, {
+                    'set': mapping_args,
+                    'tag_from_label': 'bar',
+                    'includes': ['nova', 'neutron'],
+                    'push_destination': True,
+                    'modify_role': 'add-foo-plugin',
+                    'modify_only_with_source': ['kolla', 'tripleo'],
+                    'modify_vars': {'foo_version': '1.0.1'}
+
                 }]
             }
         }
@@ -980,6 +1086,19 @@ class TestPrepare(base.TestCase):
                     'push_destination': '192.0.2.1:8787'
                 }]
             },
+            {
+                'image_params': {
+                    'BarImage': 't/bar:1.0',
+                    'BazImage': 't/baz:1.0'
+                },
+                'upload_data': [{
+                    'imagename': 't/bar:1.0',
+                    'push_destination': '192.0.2.1:8787'
+                }, {
+                    'imagename': 't/baz:1.0',
+                    'push_destination': '192.0.2.1:8787'
+                }]
+            },
         ]
 
         image_params = kb.container_images_prepare_multi(env, roles_data,
@@ -999,6 +1118,7 @@ class TestPrepare(base.TestCase):
                 append_tag=mock.ANY,
                 modify_role=None,
                 modify_only_with_labels=None,
+                modify_only_with_source=None,
                 modify_vars=None,
                 mirrors={
                     'docker.io': 'http://192.0.2.2/reg/'
@@ -1022,6 +1142,31 @@ class TestPrepare(base.TestCase):
                 append_tag=mock.ANY,
                 modify_role='add-foo-plugin',
                 modify_only_with_labels=['kolla_version'],
+                modify_only_with_source=None,
+                modify_vars={'foo_version': '1.0.1'},
+                mirrors={
+                    'docker.io': 'http://192.0.2.2/reg/'
+                },
+                registry_credentials={
+                    'docker.io': {'my_username': 'my_password'}
+                },
+                multi_arch=False,
+                lock=mock_lock
+            ),
+            mock.call(
+                excludes=None,
+                includes=['nova', 'neutron'],
+                mapping_args=mapping_args,
+                output_env_file='image_params',
+                output_images_file='upload_data',
+                pull_source=None,
+                push_destination='192.0.2.1:8787',
+                service_filter=None,
+                tag_from_label='bar',
+                append_tag=mock.ANY,
+                modify_role='add-foo-plugin',
+                modify_only_with_labels=None,
+                modify_only_with_source=['kolla', 'tripleo'],
                 modify_vars={'foo_version': '1.0.1'},
                 mirrors={
                     'docker.io': 'http://192.0.2.2/reg/'
@@ -1034,7 +1179,7 @@ class TestPrepare(base.TestCase):
             )
         ])
 
-        mock_im.assert_called_once()
+        self.assertEqual(mock_im.call_count, 2)
 
         self.assertEqual(
             {
@@ -1047,15 +1192,12 @@ class TestPrepare(base.TestCase):
         )
 
     @mock.patch('tripleo_common.image.kolla_builder.container_images_prepare')
-    @mock.patch('tripleo_common.image.image_uploader.ImageUploadManager',
-                autospec=True)
-    def test_container_images_prepare_multi_dry_run(self, mock_im, mock_cip):
+    def test_container_images_prepare_multi_dry_run(self, mock_cip):
         mock_lock = mock.MagicMock()
         mapping_args = {
             'namespace': 't',
             'name_prefix': '',
             'name_suffix': '',
-            'tag': 'l',
         }
         env = {
             'parameter_defaults': {
@@ -1116,6 +1258,7 @@ class TestPrepare(base.TestCase):
                 append_tag=mock.ANY,
                 modify_role=None,
                 modify_only_with_labels=None,
+                modify_only_with_source=None,
                 modify_vars=None,
                 mirrors={},
                 registry_credentials=None,
@@ -1135,6 +1278,7 @@ class TestPrepare(base.TestCase):
                 append_tag=mock.ANY,
                 modify_role='add-foo-plugin',
                 modify_only_with_labels=['kolla_version'],
+                modify_only_with_source=None,
                 modify_vars={'foo_version': '1.0.1'},
                 mirrors={},
                 registry_credentials=None,
@@ -1142,10 +1286,117 @@ class TestPrepare(base.TestCase):
                 lock=mock_lock
             )
         ])
+        self.assertEqual(
+            {
+                'BarImage': 't/bar:1.0',
+                'BazImage': 't/baz:1.0',
+                'BinkImage': 't/bink:latest',
+                'FooImage': 't/foo:latest'
+            },
+            image_params
+        )
 
-        mock_im.assert_called_once_with(mock.ANY, dry_run=True, cleanup='full',
-                                        mirrors={}, registry_credentials=None,
-                                        multi_arch=False, lock=mock_lock)
+    @mock.patch('tripleo_common.image.kolla_builder.container_images_prepare')
+    def test_container_images_prepare_multi_tag_from_label(self, mock_cip):
+        mock_lock = mock.MagicMock()
+        mapping_args = {
+            'namespace': 't',
+            'name_prefix': '',
+            'name_suffix': '',
+            'tag': 'l',
+        }
+        mapping_args_no_tag = {
+            'namespace': 't',
+            'name_prefix': '',
+            'name_suffix': '',
+        }
+        env = {
+            'parameter_defaults': {
+                'ContainerImagePrepare': [{
+                    'set': mapping_args_no_tag,
+                    'tag_from_label': 'foo',
+                }, {
+                    'set': mapping_args,
+                    'tag_from_label': 'bar',
+                    'excludes': ['nova', 'neutron'],
+                    'push_destination': '192.0.2.1:8787',
+                    'modify_role': 'add-foo-plugin',
+                    'modify_only_with_labels': ['kolla_version'],
+                    'modify_vars': {'foo_version': '1.0.1'},
+                    'modify_append_tag': 'modify-123'
+                }]
+            }
+        }
+        roles_data = []
+        mock_cip.side_effect = [
+            {
+                'image_params': {
+                    'FooImage': 't/foo:latest',
+                    'BarImage': 't/bar:latest',
+                    'BazImage': 't/baz:latest',
+                    'BinkImage': 't/bink:latest'
+                },
+                'upload_data': []
+            }, {
+                'image_params': {
+                    'BarImage': 't/bar:1.0',
+                    'BazImage': 't/baz:1.0'
+                },
+                'upload_data': [{
+                    'imagename': 't/bar:1.0',
+                    'push_destination': '192.0.2.1:8787'
+                }, {
+                    'imagename': 't/baz:1.0',
+                    'push_destination': '192.0.2.1:8787'
+                }]
+            },
+        ]
+
+        image_params = kb.container_images_prepare_multi(env, roles_data, True,
+                                                         lock=mock_lock)
+
+        mock_cip.assert_has_calls([
+            mock.call(
+                excludes=None,
+                includes=None,
+                mapping_args=mapping_args_no_tag,
+                output_env_file='image_params',
+                output_images_file='upload_data',
+                pull_source=None,
+                push_destination=None,
+                service_filter=None,
+                tag_from_label='foo',
+                append_tag=mock.ANY,
+                modify_role=None,
+                modify_only_with_labels=None,
+                modify_only_with_source=None,
+                modify_vars=None,
+                mirrors={},
+                registry_credentials=None,
+                multi_arch=False,
+                lock=mock_lock
+            ),
+            mock.call(
+                excludes=['nova', 'neutron'],
+                includes=None,
+                mapping_args=mapping_args,
+                output_env_file='image_params',
+                output_images_file='upload_data',
+                pull_source=None,
+                push_destination='192.0.2.1:8787',
+                service_filter=None,
+                tag_from_label=None,
+                append_tag=mock.ANY,
+                modify_role='add-foo-plugin',
+                modify_only_with_labels=['kolla_version'],
+                modify_only_with_source=None,
+                modify_vars={'foo_version': '1.0.1'},
+                mirrors={},
+                registry_credentials=None,
+                multi_arch=False,
+                lock=mock_lock
+            )
+        ])
 
         self.assertEqual(
             {
