@@ -23,11 +23,13 @@ import warnings
 import yaml
 
 import jinja2
+from oslo_utils import uuidutils
 
 from tripleo_common import constants
 from tripleo_common.utils.safe_import import git
 
-warnings.filterwarnings('once')
+warnings.filterwarnings('once', category=DeprecationWarning)
+warnings.filterwarnings('once', category=UserWarning)
 
 
 class Config(object):
@@ -115,33 +117,43 @@ class Config(object):
         return os.fdopen(
             os.open(path, os.O_WRONLY | os.O_CREAT | os.O_TRUNC, 0o600), 'w')
 
-    def _write_tasks_per_step(self, tasks, role, filepath, step):
+    def _write_tasks_per_step(self, tasks, filepath, step, strict=False):
 
-        def step_in_task(task, step):
+        def step_in_task(task, step, strict):
             whenexpr = task.get('when', None)
             if whenexpr is None:
-                # If no step is defined, it will be executed for all
-                # steps.
-                return True
+                if not strict:
+                    # If no step is defined, it will be executed for all
+                    # steps if strict is false
+                    return True
+                else:
+                    # We only want the task with the step defined.
+                    return False
             if not isinstance(whenexpr, list):
                 whenexpr = [whenexpr]
-            for w in whenexpr:
-                # "when" accepts booleans and regex only work for strings
-                if not isinstance(w, six.string_types):
-                    continue
-                # remove all spaces to make the regex match easier
-                w = re.sub(r'\s+', '', w)
-                # make \|int optional incase forgotten; use only step digit:
-                # ()'s around step|int are also optional
-                match = re.search(r'\(?step(\|int)?\)?==([0-9]+)$', "%s" % w)
-                if match:
-                    if match.group(2) == str(step):
-                        return True
-                    else:
-                        return False
+
+            # Filter out boolean value and remove blanks
+            flatten_when = "".join([re.sub(r'\s+', '', x)
+                                    for x in whenexpr
+                                    if isinstance(x, six.string_types)])
+            # make \|int optional incase forgotten; use only step digit:
+            # ()'s around step|int are also optional
+            steps_found = re.findall(r'\(?step(?:\|int)?\)?==(\d+)',
+                                     flatten_when)
+            if steps_found:
+                if str(step) in steps_found:
+                    return True
+                else:
+                    return False
+
+            # No step found
+            if strict:
+                return False
             return True
 
-        tasks_per_step = [task for task in tasks if step_in_task(task, step)]
+        tasks_per_step = [task for task in tasks if step_in_task(task,
+                                                                 step,
+                                                                 strict)]
         with self._open_file(filepath) as conf_file:
             yaml.safe_dump(tasks_per_step, conf_file, default_flow_style=False)
         return tasks_per_step
@@ -211,6 +223,12 @@ class Config(object):
         network_config = self.get_network_config_data(stack)
         for server, config in network_config.items():
             if server in server_roles:
+                if config is None or not uuidutils.is_uuid_like(config):
+                    # config should be a valid heat resource id
+                    err_msg = ('Invalid network config for role %s. Please '
+                               'check the network config templates used.' %
+                               server_roles[server])
+                    raise ValueError(err_msg)
                 server_deployment_dir = os.path.join(
                     config_dir, server_roles[server], server)
                 self._mkdir(server_deployment_dir)
@@ -258,12 +276,14 @@ class Config(object):
                     # run per step.
                     # We include it here to allow the CI to pass until THT
                     # changed is not merged.
-                    if config in constants.PER_STEP_TASKS:
-                        for i in range(constants.DEFAULT_STEPS_MAX):
+                    if config in constants.PER_STEP_TASKS.keys():
+                        for i in range(len(constants.PER_STEP_TASKS[config])):
                             filepath = os.path.join(role_path, '%s_step%s.yaml'
                                                     % (config, i))
-                            self._write_tasks_per_step(role[config], role_name,
-                                                       filepath, i)
+                            self._write_tasks_per_step(
+                                role[config],
+                                filepath,
+                                i, constants.PER_STEP_TASKS[config][i])
 
                     try:
                         data = role[config]
@@ -279,6 +299,16 @@ class Config(object):
 
         role_config = self.get_role_config()
         for config_name, config in six.iteritems(role_config):
+
+            # External tasks are in RoleConfig and not defined per role.
+            # So we don't use the RoleData to create the per step playbooks.
+            if config_name in constants.EXTERNAL_TASKS:
+                for i in range(constants.DEFAULT_STEPS_MAX):
+                    filepath = os.path.join(config_dir,
+                                            '%s_step%s.yaml'
+                                            % (config_name, i))
+                    self._write_tasks_per_step(config, filepath, i)
+
             conf_path = os.path.join(config_dir, config_name)
             # Add .yaml extension only if there's no extension already
             if '.' not in conf_path:
